if(!dojo._hasResource["com.ibm.mm.builder.DelegatedDeferredImpl"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.builder.DelegatedDeferredImpl"] = true;
/**
 * Provides enhanced Deferred implementations for dealing with situations where data
 * is loaded from one place by multiple consumers.  Each consumer gets its own Deferred
 * implementation, but the data loading can be performed by a single Deferred under the
 * covers.
 * @author Ben
 */
dojo.provide("com.ibm.mm.builder.DelegatedDeferredImpl");

dojo.require("com.ibm.mm.builder.DeferredImpl");

/**
 * A DelegatorDeferredImpl is a deferred that allows DelegatedDeferredImpls to piggy back on top of it
 * so that its loading operation will fire the callbacks of all DelegatedDeferredImpls that are attached
 * to it.  This provides a mechanism for a single pipe to load data and then multiplex the output to 
 * multiple other outputs.
 */
dojo.declare("com.ibm.mm.builder.DelegatorDeferredImpl", com.ibm.mm.builder.DeferredImpl, {
	
	constructor: function() {
		this._delegates = [];
		this._finishedData = [];
		var me = this;
		this.finishedCallback2 = function(result, status, params){
			if(!me._restarted || !me._finished) {
				// Only call the callbacks if we were never finished before, or if we've been finished
				// already without having been restarted (like if a particular caller calls the callback
				// multiple times at the end of an async operation; unlikely but possible)
				me._finishedData = [result, status, params];
				me._finished = true;
				while(me._delegates.length > 0) {
					me._fireDelegate(me._delegates.shift());
				}
				return me._finish.apply(me, arguments);
			}
		};
	},
	_finish: function(result, status, params) {
		if(this._hiddenCallback) {
			return this._hiddenCallback.apply(this, arguments);
		}
	},
	
	setFinishedCallback: function(callback, parameters){
		this._hiddenCallback = callback;
		this.finishedCallbackParameters2 = parameters;
		return this;
	},
	/**
	 * Important semantics to note here... cannot be started twice
	 */
	start: function(sync){
		if(this._started) {
			return;
		}
		this._started = true;
		return this.inherited(arguments);
	},
	/**
	 * This allows a caller to "restart" this DelegatorDeferredImpl in sync mode if it is currently processing
	 * in async mode and hasn't finished yet.
	 */
	_restart: function(sync) {
		if(sync && this._started && !this._finished) {
			// we started in async mode but need to restart in sync mode
			this._restarted = true;
			this._started = false;
			return this.start(true);
		}
	},
	_fireDelegate: function(dfd) {
		var data = this._finishedData,
			cbk = dfd.getFinishedCallback(),
			parms = dfd.getFinishedCallbackParameters();
		if(cbk) {
			// Do NOT catch errors so that they propagate to the top
			// Side effect is the potential to stop calling callbacks if an error occurs in one of them
			return cbk(data[0], data[1], parms);
		}
	},
	/**
	 * Adds a DelegatedDeferredImpl to this DelegatorDeferredImpl's delegate chain
	 */
	addDelegate: function(dfd) {
		this._delegates.push(dfd);
		dfd.delegator = this;
		if(this._finished) {
			this._fireDelegate(this._delegates.shift());
		}
		return this;
	}
});


/**
 * A DelegatedDeferredImpl is a deferred which actually piggy backs on top of an existing DelegatorDeferredImpl.
 * These are created without a start function and are then added to an existing DelegatorDeferredImpl via its
 * addDelegate API.  When the DelegatorDeferredImpl finishes, this Deferred is fired as well with the same results
 * so that multiple Deferreds can be passed to consuming code while relying on a single pipe for the actual
 * data.
 */
dojo.declare("com.ibm.mm.builder.DelegatedDeferredImpl", com.ibm.mm.builder.DeferredImpl, {
	
	delegator: null,
	
	setFinishedCallback: function(callback, parameters){
		var me = this;
		this.finishedCallback2 = function(result, status, params){
			if(me._started) {
				me._result = result;	// store this for sync mode
				// Only call our callback if we've been started already
				return callback.apply(this, arguments);
			}
		};
		this.finishedCallbackParameters2 = parameters;
		return this;
	},
	
	/**
	 * Important note on side effects here... starting a DelegatedDeferredImpl will implicitly start the underlying
	 * delegator if it hasn't been started yet.
	 */
	start: function(sync){
		if(!this.delegator) {
			// if we have no delegator, just escape immediately
			return;
		}
		this._started = true;
		var mode = (sync || typeof(sync) === "undefined") ? true : false;
		if(this.delegator._finished && this.finishedCallback2) {
			// the delegator finished already, add us back to it to be automatically fired
			this.delegator.addDelegate(this);
			return this._result;	// for sync support
		}
		else if(mode && this.delegator._started && !this.delegator._finished) {
			// the delegator was started in async mode, hasn't finished, and we're starting in sync mode
			return this.delegator._restart(mode);
		}
		else if(!this.delegator._started) {
			// the delegator hasn't started yet
			return this.delegator.start(mode);
		}
	}
});

}
