if(!dojo._hasResource["com.ibm.mm.builder.WidgetCreator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.mm.builder.WidgetCreator"] = true;
/**
 * This class is used to create the span microformat for a iWidget
 * All the information should be passed in by a com.ibm.mm.builder.widget.PaletteEntry object
 * 
 * @author Marshall (shiwcsdl@cn.ibm.com)
 */
dojo.provide("com.ibm.mm.builder.WidgetCreator");

dojo.declare("com.ibm.mm.builder.WidgetCreator",null,{
	
	widgetSkin:null,
	
	widgetDefXML:null,
	
	onNewWire:null,
	
	widgetType:null,
	
	uid:null,
	
	widgetTitle:null,
	
	systemWidget:null,
	
	widgetParam:null,
	
	configPanelNodeName:null,

	/**
	 * @constructor
	 * @param {String} uid the unique id of this iWidget instance
	 * @param {Object} inlineItemObj is an object in type of com.ibm.mm.builder.widget.PaletteEntry.
	 * this object should take all the information which is needed to create the microformat of the iWidget.
	 */
	constructor: function (_uid,widgetMetaData,itemset,widgetMap) {
		this.uid = _uid;
		this.itemSet = itemset;
		
		//if (typeof widgetMetaData != 'undefined'){
		if (widgetMetaData) {
			this.widgetDefXML = widgetMetaData.getDefinitionURL(false);
			var onnewwire = widgetMetaData.getMetadata("onNewWire");
			if (onnewwire == ""){
				this.onNewWire = "null";
			}else{
				this.onNewWire = onnewwire;
			}
			// Reading widgettype for determing other types of widgets such open social and open ajax.
			var widgetType = widgetMetaData.getMetadata("widgetType");
			if (widgetType == ""){
				this.widgetType = null;
			}else{
				this.widgetType = widgetType;
			}
			this.widgetTitle = com.ibm.mm.builder.utils.enablerUtil.getNodeTitle(widgetMetaData);
			this.systemWidget = false;
			this.widgetParam = this._jsonObj(widgetMetaData.getMetadata("widgetParam"));
			this.istrusted = com.ibm.mm.builder.utils.iwidgetUtil.isTrustedWidget(widgetMetaData.getID());
			this.autowire = widgetMetaData.getMetadata("com.ibm.mashups.builder.autoWiringEnabled");
		} else {
			this.widgetDefXML = widgetMap.definitionURL;
			var onnewwire = widgetMap.onNewWire;
			if (onnewwire == ""){
				this.onNewWire = "null";
			}else{
				this.onNewWire = onnewwire;
			}
			// Reading widgettype for determing other types of widgets such open social and open ajax.

			this.widgetType = null;			

			this.widgetTitle = widgetMap.label;
			this.systemWidget = false;
			this.widgetParam = this._jsonObj(widgetMap.widgetParam);
			this.istrusted = widgetMap.sandbox_user == "true" ? false : true;
			this.autowire = widgetMap.onAutoWire;
		}
	},
	
	
	_jsonObj: function(paramStr) {
		return (paramStr)? dojo.fromJson(paramStr): {};
	},
	
	/**
	 * set the skin type, it should be "none" or "shadow" or "blue"
	 * @param {String} _widgetSkin
	 */
	setWidgetSkin:function(_widgetSkin){
		this.widgetSkin = _widgetSkin;
	},
	
	/**
	 * return the configPanelNodeName
	 */
	getConfigPanelNodeName:function(){
		return this.configPanelNodeName;
	},
	
	/**
	 * create the root span string, this is the start of the span
	 */
	createRootSpan:function(){
		var rootSpan = document.createElement("span");
		
		// other widget types for openSocial will be set here, by default we'll create the mm_iWidget
		if (this.widgetType){
			dojo.addClass(rootSpan, this.widgetType);
		}else{
			//dojo.addClass(rootSpan, com.ibm.mm.builder.constant.mmIWidget);
			dojo.addClass(rootSpan, com.ibm.mm.builder.constant.iwIWidget);
			dojo.addClass(rootSpan, com.ibm.mm.builder.constant.iWStandalone);
		}

		rootSpan.setAttribute(com.ibm.mm.builder.constant.widget_span_attr_id,""+this.uid);
		rootSpan.setAttribute(com.ibm.mm.builder.constant.widget_span_attr_skin,this.widgetSkin);	
		com.ibm.mm.builder.utils.iWidgetSpecUtil.setWidgetDefXML(rootSpan,this.widgetDefXML);		
		if (!this.istrusted){
			rootSpan.setAttribute("sandbox","true");
		}
		if (this.autowire == "true"){
			rootSpan.setAttribute("autowiringenabled","true");
		}	
		return rootSpan;
	},

	/**
	 * create the parameter string
	 */	
	createParamsElement: function(){
		var param = null;
		if (this.widgetParam && this.widgetParam.length) {
			param = com.ibm.mm.builder.utils.iWidgetSpecUtil.newItemSetSpan();
			dojo.forEach(this.widgetParam, function(item){
				com.ibm.mm.builder.utils.iWidgetSpecUtil.newItemSetKeyValue(param, item.itemName, item.itemValue);
			});
		}
		if (this.itemSet) {
			param = param || com.ibm.mm.builder.utils.iWidgetSpecUtil.newItemSetSpan();
			for (var key in this.itemSet) {
				if (Object.prototype.hasOwnProperty.call(this.itemSet, key)) {
					var ii = dojo.query("[href='#" + key + "']", param);
					if (ii && ii[0]) {
						ii[0].innerHTML = this.itemSet[key];
					}
					else {
						com.ibm.mm.builder.utils.iWidgetSpecUtil.newItemSetKeyValue(param, key, this.itemSet[key]);
					}
				}
			}
		}
		return param;
	},		

	/**
	 * call this method to get the microformat of a widget
	 */					
	getWidget:function(){
		var param = this.createParamsElement();
		var root = this.createRootSpan();
		if (param!==null){
			root.appendChild(param);
		}
		return root;
	}
	
});

}
