dojo.provide("com.ibm.pb.dnd.layout.LayoutMashupColumnSource");

dojo.require("com.ibm.pb.dnd.layout.LayoutColumnSource");
dojo.require("com.ibm.mm.builder.dnd.QuickWireHelper");

dojo.declare("com.ibm.pb.dnd.layout.LayoutMashupColumnSource", 
		[com.ibm.pb.dnd.layout.LayoutColumnSource, com.ibm.mm.builder.dnd.QuickWireHelper], {

	layoutParser: com.ibm.pb.layout.LayoutParser,

	onDndCancel: function() {
		this.inherited(arguments);
		this.destroyAllQuickWireTarget();
	},
	
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		// Customization: add topPos
		if(this.isDragging && this.targetState == "Disabled"){ return; }
		dojo.dnd.Source.superclass.onMouseMove.call(this, e);
		var m = dojo.dnd.manager();
		if(!this.isDragging){
			if(this.mouseDown && this.isSource &&
					(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay)){
				var nodes = this.getSelectedNodes();
				if(nodes.length){
					m.startDrag(this, nodes, this.copyState(dojo.isCopyKey(e), true));
				}
			}
		}
		if(this.isDragging){
			// calculate before/after
			var before = false;
			var topPos = -1;
			if(this.current){
				if(!this.targetBox || this.targetAnchor != this.current){
					this.targetBox = dojo.position(this.current, true);
				}
				if(this.horizontal){
					topPos = e.pageX - this.targetBox.x;
					before = topPos < (this.targetBox.w / 2);
				}else{
					topPos = e.pageY - this.targetBox.y;
					before = topPos < (this.targetBox.h / 2);
				}
			}
			if(this.current != this.targetAnchor || before != this.before){
				this._markTargetAnchor(before, topPos);
				m.canDrop(!this.current || m.source != this || !(this.current.id in this.selection));
			}
		}
	},
	
	_markTargetAnchor: function(before, topPos){
		// summary:
		//		assigns a class to the current target anchor based on "before" status
		// before: Boolean
		//		insert before, if true, after otherwise
		// Customization: add topPos
		if(this.current == this.targetAnchor && this.before == before){ return; }
		if(this.targetAnchor){
			this._removeItemClass(this.targetAnchor, this.before ? "Before" : "After");
		}
		this.targetAnchor = this.current;
		this.targetBox = null;
		this.before = before;
		if(this.targetAnchor){
			this._addItemClass(this.targetAnchor, this.before ? "Before" : "After", topPos);
		}
	},
	
	_addItemClass: function(node, type, topPos){
		// summary: adds a class with prefix "dojoDndItem" and create ghost
		// node: Node: a node
		// type: String: a variable suffix for a class name
		if(this.current == node && this.before == type){ return; }
		if(this.isDragging){
			var _id = this.layoutParser.extractID(node);
			if(topPos == undefined) {
				// we are dragging from the content shelf
				this.createGhost(node, type);
			} else if(topPos > this.quickWireHandlerHeight) {
				// we are not dragging on the handler
				this.createGhost(node, type);
				this.destroyQuickWireTarget(_id);
			} else {
				// we are dragging on the handler
				if(this.quickWireHandlers[_id]) {
					// do nothing
				} else {
					this.createQuickWireTarget(_id);
				}
			}
		}
	}
});

com.ibm.pb.dnd.layout.LayoutMashupColumnSource.creator = 
	dojo.hitch(com.ibm.pb.dnd.layout.LayoutMashupColumnSource, 
			com.ibm.pb.dnd.layout.LayoutColumnSource.creator);

com.ibm.pb.dnd.layout.LayoutMashupColumnSource.withHandles = true;